/******************************************************************************
Copyright (C) 2007, Nstor Daniel Altamirano, Bs. As., Argentina
E-mail: ndaltamirano@gmail.com

			**********SOFTWARE UNDER GNU GENERAL PUBLIC LICENCE***********
This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place, Suite 330, Boston, MA 02111-1307 USA
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <curses.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define xi 4		// minimum row
#define xf xi+16	// maximum row
#define yi 15		// minimum column
#define yf yi+50	// maximum column

//***********************************************************************
void show (int fila, unsigned char *texto)
{
    int i=0;
    while (texto[i])
        i++;
    mvprintw (fila,((yf-yi-i)/2)+yi,(char *)texto);
}
//***********************************************************************
void background ()
{
    unsigned int f, c;
    for (f=xi; f<=xf ;f++)
        for (c=yi; c<=yf ;c++)
            mvprintw (f,c," ");
}
//***********************************************************************
void menu_show ()
{
    background ();
    show (xi+1,"CONTROLADOR DE MOTOR PASO A PASO");
    show (xi+2,"Opciones");
    show (xi+4,"0 - Movimiento horario");
    show (xi+5,"1 - Movimiento anti-horario");
    show (xi+6,"2 - Seleccionar secuencia normal");
    show (xi+7,"3 - Seleccionar secuencia wave drive");
    show (xi+8,"4 - Seleccionar secuencia half step");
    show (xi+9,"5 - Tiempo entre pasos [ciclos-procesador/seg]");
    show (xi+10,"6 - Mover un paso");
    show (xi+11,"7 - Iniciar secuencia seleccionada");
    show (xi+12,"8 - Detener secuencia");
    show (xi+14,"9 - Salir");
    show (xf-1,"Seleccione una opcion [0-9]:");
    refresh ();
}
//***********************************************************************
void error ()
{
    background();
    show (xi - 1,"Opcion invalida. Revise si CAPSLOCK esta activada.");
    show (xi - 2,"(presione cualquier tecla para continuar)");
    getch();
}
//**********************************************************************
void error_sec_no_def ()
{
    background();
    show (xi + 1,"ERROR: No hay una secuencia definida.");
    show (xi - 2,"(presione cualquier tecla para continuar)");
    getch();
}
//**********************************************************************
void error_sec_run ()
{
    background();
    show (xi + 1,"ERROR: Hay una secuencia en ejecucion.");
    show (xi - 2,"(presione cualquier tecla para continuar)");
    getch();
}
//**********************************************************************
void error_no_sec_run ()
{
    background();
    show (xi + 1,"ERROR: No hay ninguna secuencia en ejecucion.");
    show (xi - 2,"(presione cualquier tecla para continuar)");
    getch();
}
//**********************************************************************
int make_integer()
{
    int x, number;
    number = 0;
    do
    {
        x = 0;
        x = getch();
        if ((x >= 48) && (x <= 57))
        {
            number = number * 10 + (x - 48);
        }
    }
    while ((x != 10) && (number < 1000));

    if (number > 1000)
        number = 100;
    return number;
}
//***********************************************************************
void set_time_steps (int file)
{
    int aux;
    background();
    refresh();
    show (xi+6,"Por favor, ingrese los ciclos entre cada paso [1-1000]: ");
    refresh();
    aux = make_integer();
    ioctl(file, 90, aux);
}
//***********************************************************************
int main (int argc,char ** argv)
{
    int file, option, result;
    int exit = option = result = 0;

    initscr(); /* inicializar la librera curses */
    keypad(stdscr, TRUE); /* permitir el mapeo de teclado */
    //cbreak(); /* leer caracteres 1 cada vez, no esperarlos */
    //noecho(); /* no hacer el eco de entrada */
    if (has_colors())
    {
        start_color();
        init_pair(1, COLOR_BLACK, COLOR_WHITE);
        init_pair(2, COLOR_BLACK, COLOR_WHITE);
        init_pair(3, COLOR_BLACK, COLOR_WHITE);
    };
    attrset(COLOR_PAIR(1));
    file = open("/dev/stmtrdrv",O_RDONLY);
    if (!file)
    {
        show (((xf-xi)/2)+xi, "The device is not available.");
        show (xf-2,"(Press a key to terminate program.)");
        getch();
        return (-1);
    }
    else
    {
        show (((xf - xi) / 2) + xi , "Dispositivo cargado exitosamente.");
        show (xf-2,"(presione cualquier tecla para continuar)");
        getch();
    }
    attrset(COLOR_PAIR(2));
    do
    {
        background();
        refresh();
        menu_show();
        option = getch();
        if ((option >= 48) && (option <= 57))
        {
            option = option - 48;
            switch (option)
            {
            case 0: // Set clockwise motion
                ioctl(file, 10, 0);
                break;
            case 1: // Set anti-clockwise motion
                ioctl(file, 20, 0);
                break;
            case 2: // Select normal secuence
                ioctl(file, 30, 0);
                break;
            case 3: // Select wave drive secuence
                ioctl(file, 40, 0);
                break;
            case 4: // Select half step secuence
                ioctl(file, 50, 0);
                break;
            case 5: // Set delay between steps [msec]
                set_time_steps(file);
                break;
            case 6: // Move one step
                result = ioctl(file, 60, 0);
                break;
            case 7: // Start selected secuence
                result = ioctl(file, 70, 0);
                break;
            case 8: // Stop engine
                result = ioctl(file, 80, 0);
                break;
            case 9: // Exit
                ioctl(file, 80, 0);
                exit = 1;
                break;
            default:
                error();
                break;
            }
            switch (result)
            {
            case 1: //The secuence hasnt been properly defined.
                error_sec_no_def();
                break;
            case 2: //Theres a secuence running.
                error_sec_run();
                break;
            case 3: //Theres no secuence running.
                error_no_sec_run();
                break;
            default:
                break;
            }
            result = 0;
        }
    }
    while (!exit);
    attrset(COLOR_PAIR(3));
    background();
    refresh();
    endwin();
    close(file);
}
