/******************************************************************************
Copyright (C) 2007, Nstor Daniel Altamirano, Bs. As., Argentina
E-mail: ndaltamirano@gmail.com

			**********SOFTWARE UNDER GNU GENERAL PUBLIC LICENCE***********
This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place, Suite 330, Boston, MA 02111-1307 USA
*******************************************************************************/

#include "stmtrdrv.h"

void ctrl_motor_tim()
{
    down(&stop_mutex);
    if (stopped == 0)
    {
        if (norm_sec == 1)
        {
            outb(norm_secuence[position], port);
            switch (clockwise)
            {
            case 0:
                position++;
                if (position > NWSEC_MAX)
                    position = 0;
                break;
            case 1:
                position--;
                if (position < 0)
                    position = NWSEC_MAX;
                break;
            }
        }
        else if (wdrive_sec == 1)
        {
            outb(wdrv_secuence[position],port);
            switch (clockwise)
            {
            case 0:
                position++;
                if (position > NWSEC_MAX)
                    position = 0;
                break;
            case 1:
                position--;
                if (position < 0)
                    position = NWSEC_MAX;
                break;
            }
        }
        else if (hstep_sec == 1)
        {
            outb(hstep_secuence[position],port);
            switch (clockwise)
            {
            case 0:
                position++;
                if (position > HSTEP_MAX)
                    position = 0;
                break;
            case 1:
                position--;
                if (position < 0)
                    position = HSTEP_MAX;
                break;
            }
        }
        init_timer(&ctrl_tim);
        ctrl_tim.function = ctrl_motor_tim;
        ctrl_tim.expires = jiffies + delay;
        add_timer(&ctrl_tim);
    }
    up(&stop_mutex);
}

//Initializes module
int init_module()
{
    int error = 0;
    error = check_region(port,range);
    if (error < 0)
    {
        printk("Port region busy.\n");
        return error;
    }
    request_region(port,range,"stmtrdrv");
    printk("Module registered on: region %i, address space %i.\n",port,range);

    error = register_chrdev(major,"stmtrdrv",&motor_fops);
    if (error < 0)
    {
        printk("Access to major %i denied.\n",major);
        release_region(port,range);
        printk("Port released.\n");
        return error;
    }
    init_MUTEX(&stop_mutex);
    return 0;
}

int motor_open(struct inode *inode, struct file *filp)
{
    MOD_INC_USE_COUNT;
    return 0;
}

int motor_release(struct inode *inode, struct file *filp)
{
    MOD_DEC_USE_COUNT;
    return 0;
}

int motor_write(struct file *filp, const char *buf, size_t count, loff_t *t_pos)
{
    return 0;
}

int motor_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
    switch (cmd)
    {
    case 10: //Clockwise
        down(&stop_mutex);
        clockwise = 0;
        up(&stop_mutex);
        return 0;
    case 20: //Anti-Clockwise
        down(&stop_mutex);
        clockwise = 1;
        up(&stop_mutex);
        return 0;
    case 30: //Normal secuence selected
        down(&stop_mutex);
        if (norm_sec == 0)
        {
            if (wdrive_sec == 1)
            {
                int i = 0;
                int j = 0;
                while ((i<=HSTEP_MAX) && (hstep_secuence[i]!=wdrv_secuence[position]))
                    i++;
                switch (clockwise)
                {
                case 0:
                    i++;
                    break;
                case 1:
                    if (i == 0)
                        i = HSTEP_MAX;
                    else
                        i--;
                    break;
                }
                outb(hstep_secuence[i], port);
                while ((j<=NWSEC_MAX) && (norm_secuence[j]!=hstep_secuence[i]))
                    j++;
                position = j;
            }
            else if (hstep_sec == 1)
            {
                int i = 0;
                if ((position % 2) == 0)
                {
                    switch (clockwise)
                    {
                    case 0:
                        position++;
                        break;
                    case 1:
                        if (position == 0)
                            position = HSTEP_MAX;
                        else
                            position--;
                        break;
                    }
                }
                while ((i<=NWSEC_MAX) && (norm_secuence[i]!=hstep_secuence[position]))
                    i++;
                position = i;
            }
            norm_sec = 1;
            wdrive_sec = hstep_sec = 0;
        }
        up(&stop_mutex);
        return 0;
    case 40: //Wave Drive secuence selected
        down(&stop_mutex);
        if (wdrive_sec == 0)
        {
            if (norm_sec == 1)
            {
                int i = 0;
                int j = 0;
                while ((i<=HSTEP_MAX) && (hstep_secuence[i]!=norm_secuence[position]))
                    i++;
                switch (clockwise)
                {
                case 0:
                    if (i == HSTEP_MAX)
                        i = 0;
                    else
                        i++;
                    break;
                case 1:
                    i--;
                    break;
                }
                outb(hstep_secuence[i], port);
                while ((j<=NWSEC_MAX) && (wdrv_secuence[j]!=hstep_secuence[i]))
                    j++;
                position = j;
            }
            else if (hstep_sec == 1)
            {
                int i = 0;
                if ((position % 2) != 0)
                {
                    switch (clockwise)
                    {
                    case 0:
                        if (position == HSTEP_MAX)
                            position = 0;
                        else
                            position++;
                        break;
                    case 1:
                        position--;
                        break;
                    }
                }
                while ((i<=NWSEC_MAX) && (wdrv_secuence[i]!=hstep_secuence[position]))
                    i++;
                position = i;
            }
            wdrive_sec = 1;
            norm_sec = hstep_sec = 0;
        }
        up(&stop_mutex);
        return(0);
    case 50: //Half-Step secuence selected
        down(&stop_mutex);
        if (hstep_sec == 0)
        {
            int i = 0;
            if (wdrive_sec == 1)
            {
                while ((i<=HSTEP_MAX) && (hstep_secuence[i]!=wdrv_secuence[position]))
                    i++;
                position = i;
            }
            else if (norm_sec == 1)
            {
                while ((i<=HSTEP_MAX) && (hstep_secuence[i]!=norm_secuence[position]))
                    i++;
                position = i;
            }
            hstep_sec = 1;
            wdrive_sec = norm_sec = 0;
        }
        up(&stop_mutex);
        return(0);
    case 60: //Move 1 step on the engine, according to the secuence selected.
        down(&stop_mutex);
        if (stopped == 0)
        {
            up(&stop_mutex);
            //Return value 2. There's a secuence running.
            return(2);
        }
        if (norm_sec == 1)
        {
            outb(norm_secuence[position],port);
            switch (clockwise)
            {
            case 0:
                position++;
                if (position > NWSEC_MAX)
                    position = 0;
                break;
            case 1:
                position--;
                if (position < 0)
                    position = NWSEC_MAX;
                break;
            }
            up(&stop_mutex);
            return(0);
        }
        else if (wdrive_sec == 1)
        {
            outb(wdrv_secuence[position],port);
            switch (clockwise)
            {
            case 0:
                position++;
                if (position > NWSEC_MAX)
                    position = 0;
                break;
            case 1:
                position--;
                if (position < 0)
                    position = NWSEC_MAX;
                break;
            }
            up(&stop_mutex);
            return(0);
        }
        else if (hstep_sec == 1)
        {
            outb(hstep_secuence[position],port);
            switch (clockwise)
            {
            case 0:
                position++;
                if (position > HSTEP_MAX)
                    position = 0;
                break;
            case 1:
                position--;
                if (position < 0)
                    position = HSTEP_MAX;
                break;
            }
            up(&stop_mutex);
            return(0);
        }
        up(&stop_mutex);
        //Return value 1. The secuence hasn't been properly defined.
        return(1);
    case 70: //Start selected secuence
        down(&stop_mutex);
        if ((norm_sec == 1) || (wdrive_sec == 1) || (hstep_sec == 1))
        {
            if (stopped == 1)
            {
                stopped = 0;
                init_timer(&ctrl_tim);
                ctrl_tim.function = ctrl_motor_tim;
                ctrl_tim.expires = jiffies + delay;
                up(&stop_mutex);
                add_timer(&ctrl_tim);
                return(0);
            }
            else
            {
                up(&stop_mutex);
                //Return value 2. There's a secuence running.
                return(2);
            }
        }
        up(&stop_mutex);
        //Return value 1. The secuence hasn't been properly defined.
        return(1);
    case 80: //Stop motor
        down(&stop_mutex);
        if (stopped == 0)
        {
            del_timer(&ctrl_tim);
            stopped = 1;
            up(&stop_mutex);
            outb(0x00,port);
            //The engine has been stopped.
            return(0);
        }
        up(&stop_mutex);
        //Return value 3. The engine isn't running on a secuence.
        return(3);
    case 90: //Set timer delay between steps
        down(&stop_mutex);
        if (arg <= 0) //We cant allow negative or zero delay, can we?
            arg = 1;
        delay = arg;
        up(&stop_mutex);
        return(0);
    default:
        //Invalid option
        return(4);
    }
}

void cleanup_module(void)
{
    down(&stop_mutex);
    if (stopped == 0)
    {
        del_timer(&ctrl_tim);
        printk("Engine was running. Turned off.\n");
    }
    up(&stop_mutex);
    outb(0x00,port);
    unregister_chrdev(major,"stmtrdrv");
    printk("Major unregistered.\n");
    release_region(port,range);
    printk("Port released.\n");
}
