/******************************************************************************
Copyright (C) 2007, Nstor Daniel Altamirano, Bs. As., Argentina
E-mail: ndaltamirano@gmail.com

			**********SOFTWARE UNDER GNU GENERAL PUBLIC LICENCE***********
This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place, Suite 330, Boston, MA 02111-1307 USA
*******************************************************************************/

#ifndef MODULE
#define MODULE
#endif
#ifndef __KERNEL__
#define __KERNEL__
#endif

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/param.h>
#include <linux/sched.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Nestor Daniel Altamirano");
MODULE_DESCRIPTION("Stepper Motor Driver");
MODULE_SUPPORTED_DEVICE("stmtrdrv");

//Memory buffer's size variables
#define HSTEP_MAX 7
#define NWSEC_MAX 3

//Parallel Port variables
int major = 253;
int range = 3;
int port = 0x378;
unsigned long delay = 1;

//Secuence and position variables
int norm_sec = 0;
int wdrive_sec = 0;
int hstep_sec = 0;
int stopped = 1;
int clockwise = 1;
int position = 0;

//Memory buffers
unsigned char norm_secuence[] = {0x03,0x06,0x0C,0x09};
unsigned char wdrv_secuence[] = {0x01,0x02,0x04,0x08};
unsigned char hstep_secuence[] = {0x01,0x03,0x02,0x06,0x04,0x0C,0x08,0x09};

//Mutex semaphore
struct semaphore stop_mutex;

//Timer between runs
struct timer_list ctrl_tim;

//Driver input parameters, used by Parallel Port variables
MODULE_PARM(major,"i");
MODULE_PARM_DESC(major,"Major number");

MODULE_PARM(port,"i");
MODULE_PARM_DESC(port,"I/O port > 0x378 by default");

//Driver functions
int init_module(void);
int motor_open(struct inode *inode, struct file *filp);
int motor_write(struct file *filp, const char *buf, size_t count, loff_t *t_pos);
int motor_release(struct inode *inode, struct file *filp);
int motor_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);
void cleanup_module(void);

//Timer function
void ctrl_motor_tim();

struct file_operations motor_fops =
    {
    open:
        motor_open,
    write:
        motor_write,
    ioctl:
        motor_ioctl,
    release:
        motor_release,
    };
